@@ -235,7 +235,7 @@ cli_submit_request (struct rpc_clnt *rpc, void *req, call_frame_t *frame,
 
 int32_t
 cli_cmd_volume_create_parse (struct cli_state *state, const char **words,
-                             int wordcount, dict_t **options);
+                             int wordcount, dict_t **options, char **bricks);
 
 int32_t
 cli_cmd_volume_reset_parse (const char **words, int wordcount, dict_t **opt);
@@ -279,7 +279,7 @@ cli_cmd_volume_old_tier_parse (const char **words, int wordcount,
 int32_t
 cli_cmd_volume_remove_brick_parse (const char **words, int wordcount,
                                    dict_t **options, int *question,
-                                   int *brick_count);
+                                   int *brick_count, int32_t *command);
 
 int32_t
 cli_cmd_volume_replace_brick_parse (const char **words, int wordcount,
