@@ -1,3 +1,4 @@
+#!/usr/bin/python2
 #
 # Copyright (c) 2016 Red Hat, Inc. <http://www.redhat.com>
 # This file is part of GlusterFS.
@@ -8,8 +9,10 @@
 # cases as published by the Free Software Foundation.
 #
 
-GLUSTERFS_LIBEXECDIR = '${exec_prefix}/libexec/glusterfs'
+GLUSTERFS_LIBEXECDIR = '/usr/local/libexec/glusterfs'
 GLUSTERD_WORKDIR = "/var/lib/glusterd"
 
 LOCALSTATEDIR = "/var"
 UUID_FILE = "/var/lib/glusterd/glusterd.info"
+GLUSTERFS_CONFDIR = "/usr/local/etc/glusterfs"
+GCONF_VERSION = 4.0
