@@ -166,6 +166,13 @@ static struct argp_option gf_options[] = {
          "Validate the netgroups file and print it out"},
         {"print-exports", ARGP_PRINT_EXPORTS, "EXPORTS-FILE", 0,
         "Validate the exports file and print it out"},
+        {"print-xlatordir", ARGP_PRINT_XLATORDIR_KEY, 0, OPTION_ARG_OPTIONAL,
+         "Print xlator directory path"},
+        {"print-statedumpdir", ARGP_PRINT_STATEDUMPDIR_KEY, 0,
+          OPTION_ARG_OPTIONAL,
+         "Print directory path in which statedumps shall be generated"},
+        {"print-logdir", ARGP_PRINT_LOGDIR_KEY, 0, OPTION_ARG_OPTIONAL,
+         "Print path of default log directory"},
 
         {"volfile-max-fetch-attempts", ARGP_VOLFILE_MAX_FETCH_ATTEMPTS, "0",
          OPTION_HIDDEN, "Maximum number of attempts to fetch the volfile"},
@@ -196,10 +203,8 @@ static struct argp_option gf_options[] = {
          "Enables thin mount and connects via gfproxyd daemon"},
 
         {0, 0, 0, 0, "Fuse options:"},
-        {"direct-io-mode", ARGP_DIRECT_IO_MODE_KEY, "BOOL", OPTION_ARG_OPTIONAL,
-         "Use direct I/O mode in fuse kernel module"
-         " [default: \"off\" if big writes are supported, else "
-         "\"on\" for fds not opened with O_RDONLY]"},
+        {"direct-io-mode", ARGP_DIRECT_IO_MODE_KEY, "BOOL|auto", OPTION_ARG_OPTIONAL,
+         "Specify direct I/O strategy [default: \"auto\"]"},
         {"entry-timeout", ARGP_ENTRY_TIMEOUT_KEY, "SECONDS", 0,
          "Set entry timeout to SECONDS in fuse kernel module [default: 1]"},
         {"negative-timeout", ARGP_NEGATIVE_TIMEOUT_KEY, "SECONDS", 0,
@@ -249,6 +254,13 @@ static struct argp_option gf_options[] = {
          "option to specify the process type" },
         {"event-history", ARGP_FUSE_EVENT_HISTORY_KEY, "BOOL",
          OPTION_ARG_OPTIONAL, "disable/enable fuse event-history"},
+        {"reader-thread-count", ARGP_READER_THREAD_COUNT_KEY, "INTEGER",
+         OPTION_ARG_OPTIONAL, "set fuse reader thread count"},
+        {"kernel-writeback-cache", ARGP_KERNEL_WRITEBACK_CACHE_KEY, "BOOL",
+         OPTION_ARG_OPTIONAL, "enable fuse in-kernel writeback cache"},
+        {"attr-times-granularity", ARGP_ATTR_TIMES_GRANULARITY_KEY, "NS",
+         OPTION_ARG_OPTIONAL, "declare supported granularity of file attribute"
+         " times in nanoseconds"},
         {0, 0, 0, 0, "Miscellaneous Options:"},
         {0, }
 };
@@ -528,7 +540,7 @@ set_fuse_mount_options (glusterfs_ctx_t *ctx, dict_t *options)
                         goto err;
                 }
                 break;
-        case GF_OPTION_DEFERRED: /* default */
+        case GF_OPTION_DEFERRED: /* auto */
         default:
                 gf_msg_debug ("glusterfsd", 0, "fuse direct io type %d",
                               cmd_args->fuse_direct_io_mode);
@@ -600,6 +612,54 @@ set_fuse_mount_options (glusterfs_ctx_t *ctx, dict_t *options)
                         goto err;
                 }
         }
+        if (cmd_args->reader_thread_count) {
+                ret = dict_set_uint32 (options, "reader-thread-count",
+                                       cmd_args->reader_thread_count);
+                if (ret < 0) {
+                        gf_msg ("glusterfsd", GF_LOG_ERROR, 0, glusterfsd_msg_4,
+                                "failed to set dict value for key "
+                                "reader-thread-count");
+                        goto err;
+                }
+        }
+        switch (cmd_args->kernel_writeback_cache) {
+        case GF_OPTION_ENABLE:
+                ret = dict_set_static_ptr(options, "kernel-writeback-cache",
+                        "on");
+                if (ret < 0) {
+                        gf_msg ("glusterfsd", GF_LOG_ERROR, 0, glusterfsd_msg_4,
+                                "failed to set dict value for key "
+                                "kernel-writeback-cache");
+                        goto err;
+                }
+                break;
+        case GF_OPTION_DISABLE:
+                ret = dict_set_static_ptr(options, "kernel-writeback-cache",
+                        "off");
+                if (ret < 0) {
+                        gf_msg ("glusterfsd", GF_LOG_ERROR, 0, glusterfsd_msg_4,
+                                "failed to set dict value for key "
+                                "kernel-writeback-cache");
+                        goto err;
+                }
+                break;
+        case GF_OPTION_DEFERRED: /* default */
+        default:
+                gf_msg_debug ("glusterfsd", 0, "kernel-writeback-cache mode %d",
+                              cmd_args->kernel_writeback_cache);
+                break;
+        }
+        if (cmd_args->attr_times_granularity) {
+                ret = dict_set_uint32 (options, "attr-times-granularity",
+                                       cmd_args->attr_times_granularity);
+                if (ret < 0) {
+                        gf_msg ("glusterfsd", GF_LOG_ERROR, 0, glusterfsd_msg_4,
+                                "failed to set dict value for key "
+                                "attr-times-granularity");
+                        goto err;
+                }
+        }
+
 
         ret = 0;
 err:
@@ -718,7 +778,6 @@ gf_remember_backup_volfile_server (char *arg)
         glusterfs_ctx_t         *ctx = NULL;
         cmd_args_t              *cmd_args = NULL;
         int                      ret = -1;
-        server_cmdline_t        *server = NULL;
 
         ctx = glusterfsd_ctx;
         if (!ctx)
@@ -728,39 +787,15 @@ gf_remember_backup_volfile_server (char *arg)
         if(!cmd_args)
                 goto out;
 
-        server = GF_CALLOC (1, sizeof (server_cmdline_t),
-                            gfd_mt_server_cmdline_t);
-        if (!server)
-                goto out;
-
-        INIT_LIST_HEAD(&server->list);
-
-        server->volfile_server = gf_strdup(arg);
-
-        if (!cmd_args->volfile_server) {
-                cmd_args->volfile_server = server->volfile_server;
-                cmd_args->curr_server = server;
+        ret = gf_set_volfile_server_common(cmd_args, arg,
+                                           GF_DEFAULT_VOLFILE_TRANSPORT,
+                                           GF_DEFAULT_BASE_PORT);
+        if (ret) {
+                gf_log ("glusterfs", GF_LOG_ERROR,
+                        "failed to set volfile server: %s", strerror (errno));
         }
-
-        if (!server->volfile_server) {
-                gf_msg ("glusterfsd", GF_LOG_WARNING, 0, glusterfsd_msg_10,
-                        "xlator option %s is invalid", arg);
-                goto out;
-        }
-
-        list_add_tail (&server->list, &cmd_args->volfile_servers);
-
-        ret = 0;
 out:
-        if (ret == -1) {
-                if (server) {
-                        GF_FREE (server->volfile_server);
-                        GF_FREE (server);
-                }
-        }
-
         return ret;
-
 }
 
 static int
@@ -928,6 +963,18 @@ parse_opts (int key, char *arg, struct argp_state *state)
                 cmd_args->print_exports = arg;
                 break;
 
+        case ARGP_PRINT_XLATORDIR_KEY:
+                cmd_args->print_xlatordir = _gf_true;
+                break;
+
+        case ARGP_PRINT_STATEDUMPDIR_KEY:
+                cmd_args->print_statedumpdir = _gf_true;
+                break;
+
+        case ARGP_PRINT_LOGDIR_KEY:
+                cmd_args->print_logdir = _gf_true;
+                break;
+
         case ARGP_MAC_COMPAT_KEY:
                 if (!arg)
                         arg = "on";
@@ -1056,6 +1103,9 @@ parse_opts (int key, char *arg, struct argp_state *state)
                         break;
                 }
 
+                if (strcmp (arg, "auto") == 0)
+                        break;
+
                 argp_failure (state, -1, 0,
                               "unknown direct I/O mode setting \"%s\"", arg);
                 break;
@@ -1364,6 +1414,46 @@ no_oom_api:
                 argp_failure (state, -1, 0,
                               "unknown event-history setting \"%s\"", arg);
                 break;
+        case ARGP_READER_THREAD_COUNT_KEY:
+                if (gf_string2uint32 (arg, &cmd_args->reader_thread_count)) {
+                        argp_failure (state, -1, 0,
+                                      "unknown reader thread count option %s",
+                                      arg);
+                } else if ((cmd_args->reader_thread_count < 1) ||
+                           (cmd_args->reader_thread_count > 64)) {
+                        argp_failure (state, -1, 0,
+                                      "Invalid reader thread count %s. "
+                                      "Valid range: [\"1, 64\"]", arg);
+                }
+
+                break;
+
+        case ARGP_KERNEL_WRITEBACK_CACHE_KEY:
+                if (!arg)
+                        arg = "yes";
+
+                if (gf_string2boolean (arg, &b) == 0) {
+                        cmd_args->kernel_writeback_cache = b;
+
+                        break;
+                }
+
+                argp_failure (state, -1, 0,
+                              "unknown kernel writeback cache setting \"%s\"", arg);
+                break;
+        case ARGP_ATTR_TIMES_GRANULARITY_KEY:
+                if (gf_string2uint32 (arg, &cmd_args->attr_times_granularity)) {
+                        argp_failure (state, -1, 0,
+                                      "unknown attribute times granularity option %s",
+                                      arg);
+                } else if (cmd_args->attr_times_granularity > 1000000000) {
+                        argp_failure (state, -1, 0,
+                                      "Invalid attribute times granularity value %s. "
+                                      "Valid range: [\"0, 1000000000\"]", arg);
+                }
+
+                break;
+
 	}
         return 0;
 }
@@ -1481,7 +1571,7 @@ cleanup_and_exit (int signum)
         if (ctx->active)
                 trav = ctx->active->top;
         while (trav) {
-                if (should_call_fini(ctx,trav)) {
+                if (should_call_fini(ctx, trav)) {
                         THIS = trav;
                         trav->fini (trav);
                 }
@@ -1682,6 +1772,7 @@ glusterfs_ctx_defaults_init (glusterfs_ctx_t *ctx)
         cmd_args->fuse_attribute_timeout = -1;
         cmd_args->fuse_entry_timeout = -1;
 	cmd_args->fopen_keep_cache = GF_OPTION_DEFERRED;
+        cmd_args->kernel_writeback_cache = GF_OPTION_DEFERRED;
 
         if (ctx->mem_acct_enable)
                 cmd_args->mem_acct = 1;
@@ -1989,9 +2080,17 @@ parse_cmdline (int argc, char *argv[], glusterfs_ctx_t *ctx)
         /* Do this before argp_parse so it can be overridden. */
         if (sys_access (SECURE_ACCESS_FILE, F_OK) == 0) {
                 cmd_args->secure_mgmt = 1;
+                ctx->ssl_cert_depth = glusterfs_read_secure_access_file ();
         }
 
         argp_parse (&argp, argc, argv, ARGP_IN_ORDER, NULL, cmd_args);
+
+        if (cmd_args->print_xlatordir || cmd_args->print_statedumpdir ||
+            cmd_args->print_logdir) {
+                /* Just print, nothing else to do */
+                goto out;
+        }
+
         if (cmd_args->print_netgroups) {
                 /* When this option is set we don't want to do anything else
                  * except for printing & verifying the netgroups file.
@@ -2588,6 +2687,23 @@ main (int argc, char *argv[])
         if (ret)
                 goto out;
         cmd = &ctx->cmd_args;
+
+        if (cmd->print_xlatordir) {
+                /* XLATORDIR passed through a -D flag to GCC */
+                printf ("%s\n", XLATORDIR);
+                goto out;
+        }
+
+        if (cmd->print_statedumpdir) {
+                printf ("%s\n", DEFAULT_VAR_RUN_DIRECTORY);
+                goto out;
+        }
+
+        if (cmd->print_logdir) {
+                printf ("%s\n", DEFAULT_LOG_FILE_DIRECTORY);
+                goto out;
+        }
+
         if (cmd->print_netgroups) {
                 /* If this option is set we want to print & verify the file,
                  * set the return value (exit code in this case) and exit.
