@@ -303,6 +303,18 @@ cluster_writev_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
 }
 
 
+int32_t
+cluster_put_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
+                 int32_t op_ret, int32_t op_errno, inode_t *inode,
+                 struct iatt *buf, struct iatt *preparent,
+                 struct iatt *postparent, dict_t *xdata)
+{
+        FOP_CBK (put, frame, cookie, op_ret, op_errno, inode, buf, preparent,
+                 postparent, xdata);
+        return 0;
+}
+
+
 int32_t
 cluster_flush_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
                    int32_t op_ret, int32_t op_errno,
@@ -685,6 +697,20 @@ cluster_writev (xlator_t **subvols, unsigned char *on, int numsubvols,
         return cluster_fop_success_fill (replies, numsubvols, output);
 }
 
+int32_t
+cluster_put (xlator_t **subvols, unsigned char *on, int numsubvols,
+             default_args_cbk_t *replies, unsigned char *output,
+             call_frame_t *frame, xlator_t *this, loc_t *loc,
+             mode_t mode, mode_t umask, uint32_t flags, struct iovec *vector,
+             int32_t count, off_t offset, struct iobref *iobref, dict_t *xattr,
+             dict_t *xdata)
+{
+        FOP_ONLIST (subvols, on, numsubvols, replies, output, frame, put, loc,
+                    mode, umask, flags, vector, count, offset, iobref, xattr,
+                    xdata);
+        return cluster_fop_success_fill (replies, numsubvols, output);
+}
+
 int32_t
 cluster_readv (xlator_t **subvols, unsigned char *on, int numsubvols,
                default_args_cbk_t *replies, unsigned char *output,
@@ -1191,3 +1217,101 @@ cluster_entrylk (xlator_t **subvols, unsigned char *on, int numsubvols,
         loc_wipe (&loc);
         return cluster_fop_success_fill (replies, numsubvols, locked_on);
 }
+
+int
+cluster_tiebreaker_inodelk (xlator_t **subvols, unsigned char *on,
+                            int numsubvols, default_args_cbk_t *replies,
+                            unsigned char *locked_on, call_frame_t *frame,
+                            xlator_t *this, char *dom, inode_t *inode,
+                            off_t off, size_t size)
+{
+        struct gf_flock flock = {0, };
+        int i = 0;
+        int num_success = 0;
+        loc_t loc = {0};
+        unsigned char *output = NULL;
+
+        flock.l_type = F_WRLCK;
+        flock.l_start = off;
+        flock.l_len = size;
+
+        output = alloca(numsubvols);
+        loc.inode = inode_ref (inode);
+        gf_uuid_copy (loc.gfid, inode->gfid);
+        FOP_ONLIST (subvols, on, numsubvols, replies, locked_on, frame,
+                    inodelk, dom, &loc, F_SETLK, &flock, NULL);
+
+        for (i = 0; i < numsubvols; i++) {
+                if (replies[i].valid && replies[i].op_ret == 0) {
+                        num_success++;
+                        continue;
+                }
+
+                /* TODO: If earlier subvols fail with an error other
+                 * than EAGAIN, we could still have 2 clients competing
+                 * for the lock*/
+                if (replies[i].op_ret == -1 && replies[i].op_errno == EAGAIN) {
+                        cluster_fop_success_fill (replies, numsubvols,
+                                                  locked_on);
+                        cluster_uninodelk (subvols, locked_on, numsubvols,
+                                           replies, output, frame, this, dom,
+                                           inode, off, size);
+
+                        if (num_success) {
+                                FOP_SEQ (subvols, on, numsubvols, replies,
+                                         locked_on, frame, inodelk, dom, &loc,
+                                         F_SETLKW, &flock, NULL);
+                        }
+                        break;
+                }
+        }
+
+        loc_wipe (&loc);
+        return cluster_fop_success_fill (replies, numsubvols, locked_on);
+}
+
+int
+cluster_tiebreaker_entrylk (xlator_t **subvols, unsigned char *on,
+                            int numsubvols, default_args_cbk_t *replies,
+                            unsigned char *locked_on, call_frame_t *frame,
+                            xlator_t *this, char *dom, inode_t *inode,
+                            const char *name)
+{
+        int i = 0;
+        loc_t loc = {0};
+        unsigned char *output = NULL;
+        int num_success = 0;
+
+        output = alloca(numsubvols);
+        loc.inode = inode_ref (inode);
+        gf_uuid_copy (loc.gfid, inode->gfid);
+        FOP_ONLIST (subvols, on, numsubvols, replies, locked_on, frame,
+                    entrylk, dom, &loc, name, ENTRYLK_LOCK_NB, ENTRYLK_WRLCK,
+                    NULL);
+
+        for (i = 0; i < numsubvols; i++) {
+                if (replies[i].valid && replies[i].op_ret == 0) {
+                        num_success++;
+                        continue;
+                }
+                if (replies[i].op_ret == -1 && replies[i].op_errno == EAGAIN) {
+                        cluster_fop_success_fill (replies, numsubvols,
+                                                  locked_on);
+                        cluster_unentrylk (subvols, locked_on, numsubvols,
+                                           replies, output, frame, this, dom,
+                                           inode, name);
+                        if (num_success) {
+                                FOP_SEQ (subvols, on, numsubvols, replies,
+                                         locked_on, frame, entrylk, dom, &loc,
+                                         name, ENTRYLK_LOCK, ENTRYLK_WRLCK,
+                                         NULL);
+                        } else {
+                                memset (locked_on, 0, numsubvols);
+                        }
+                        break;
+                }
+        }
+
+        loc_wipe (&loc);
+        return cluster_fop_success_fill (replies, numsubvols, locked_on);
+}
