@@ -227,7 +227,7 @@ gf_proc_dump_xlator_mem_info (xlator_t *xl)
         if (!xl)
                 return;
 
-        if (!xl->mem_acct)
+        if (!xl->mem_acct || !xl->mem_acct->rec)
                 return;
 
         gf_proc_dump_add_section ("%s.%s - Memory usage", xl->type, xl->name);
@@ -262,7 +262,7 @@ gf_proc_dump_xlator_mem_info_only_in_use (xlator_t *xl)
         if (!xl)
                 return;
 
-        if (!xl->mem_acct->rec)
+        if (!xl->mem_acct || !xl->mem_acct->rec)
                 return;
 
         gf_proc_dump_add_section ("%s.%s - Memory usage", xl->type, xl->name);
@@ -515,7 +515,6 @@ gf_proc_dump_xlator_info (xlator_t *top)
 
         trav = top;
         while (trav) {
-
                 if (ctx->measure_latency)
                         gf_proc_dump_latency_info (trav);
 
