@@ -134,6 +134,7 @@ typedef struct synclock synclock_t;
 
 
 struct syncbarrier {
+        gf_boolean_t        initialized; /*Set on successful initialization*/
 	pthread_mutex_t     guard; /* guard the remaining members, pair @cond */
 	pthread_cond_t      cond;  /* waiting non-synctasks */
 	struct list_head    waitq; /* waiting synctasks */
@@ -158,6 +159,7 @@ struct syncargs {
         dict_t             *xdata;
 	struct gf_flock     flock;
         struct gf_lease     lease;
+        dict_t             *dict_out;
 
         /* some more _cbk needs */
         uuid_t              uuid;
@@ -231,7 +233,7 @@ struct syncopctx {
 	} while (0)
 
 
-#define SYNCOP(subvol, stb, cbk, op, params ...) do {                   \
+#define SYNCOP(subvol, stb, cbk, fn_op, params ...) do {                \
                 struct  synctask        *task = NULL;                   \
                 call_frame_t            *frame = NULL;                  \
                                                                         \
@@ -249,8 +251,9 @@ struct syncopctx {
                                                                         \
                 __yawn (stb);                                           \
                                                                         \
+                frame->op = get_fop_index_from_fn (subvol, fn_op);      \
                 STACK_WIND_COOKIE (frame, cbk, (void *)stb, subvol,     \
-                                   op, params);                         \
+                                   fn_op, params);                      \
                                                                         \
                 __yield (stb);                                          \
                 if (task)                                               \
@@ -539,11 +542,13 @@ int syncop_lease (xlator_t *subvol, loc_t *loc, struct gf_lease *lease,
 int syncop_ipc (xlator_t *subvol, int op, dict_t *xdata_in, dict_t **xdata_out);
 
 int syncop_xattrop (xlator_t *subvol, loc_t *loc, gf_xattrop_flags_t flags,
-                    dict_t *dict, dict_t *xdata_in, dict_t **xdata_out);
+                    dict_t *dict, dict_t *xdata_in, dict_t **dict_out,
+                    dict_t **xdata_out);
 
 int
 syncop_fxattrop (xlator_t *subvol, fd_t *fd, gf_xattrop_flags_t flags,
-                 dict_t *dict, dict_t *xdata_in, dict_t **xdata_out);
+                 dict_t *dict, dict_t *xdata_in, dict_t **dict_out,
+                 dict_t **xdata_out);
 
 int
 syncop_seek (xlator_t *subvol, fd_t *fd, off_t offset, gf_seek_what_t what,
@@ -562,4 +567,25 @@ syncop_setactivelk (xlator_t *subvol, loc_t *loc,
                      lock_migration_info_t *locklist,  dict_t *xdata_in,
                      dict_t **xdata_out);
 
+int
+syncop_put (xlator_t *subvol, loc_t *loc, mode_t mode, mode_t umask,
+            uint32_t flags, struct iovec *vector, int32_t count,
+            off_t offset, struct iobref *iobref, dict_t *xattr,
+            struct iatt *iatt, dict_t *xdata_in, dict_t **xdata_out);
+
+int
+syncop_setactivelk_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
+                        int32_t op_ret, int32_t op_errno, dict_t *xdata);
+
+int
+syncop_icreate (xlator_t *subvol, loc_t *loc, mode_t mode, dict_t *xdata_out);
+
+int
+syncop_namelink (xlator_t *subvol, loc_t *loc, dict_t *xdata_out);
+
+int
+syncop_entrylk (xlator_t *subvol, const char *volume, loc_t *loc,
+                const char *basename, entrylk_cmd cmd, entrylk_type type,
+                dict_t *xdata_in, dict_t **xdata_out);
+
 #endif /* _SYNCOP_H */
