@@ -65,6 +65,9 @@ enum gf_fop_procnum {
         GFS3_OP_LEASE,
         GFS3_OP_GETACTIVELK,
         GFS3_OP_SETACTIVELK,
+        GFS3_OP_ICREATE,
+        GFS3_OP_NAMELINK,
+        GFS3_OP_PUT,
         GFS3_OP_MAXVALUE,
 };
 
@@ -148,6 +151,8 @@ enum gf_cbk_procnum {
         GF_CBK_CHILD_DOWN,
         GF_CBK_RECALL_LEASE,
         GF_CBK_STATEDUMP,
+        GF_CBK_INODELK_CONTENTION,
+        GF_CBK_ENTRYLK_CONTENTION,
         GF_CBK_MAXVALUE,
 };
 
@@ -237,6 +242,7 @@ enum glusterd_brick_procnum {
         GLUSTERD_BRICK_BARRIER,
         GLUSTERD_NODE_BITROT,
         GLUSTERD_BRICK_ATTACH,
+        GLUSTERD_DUMP_METRICS,
         GLUSTERD_BRICK_MAXVALUE,
 };
 
@@ -319,6 +325,11 @@ enum gf_get_snapshot_info_type {
 };
 typedef enum gf_get_snapshot_info_type gf_get_snapshot_info_type;
 
+enum gf_getspec_flags_type {
+        GF_GETSPEC_FLAG_SERVERS_LIST = 1
+};
+typedef enum gf_getspec_flags_type gf_getspec_flags_type;
+
 #define GLUSTER_HNDSK_PROGRAM    14398633 /* Completely random */
 #define GLUSTER_HNDSK_VERSION    2   /* 0.0.2 */
 
@@ -332,6 +343,8 @@ typedef enum gf_get_snapshot_info_type gf_get_snapshot_info_type;
 #define GLUSTER_FOP_VERSION   330 /* 3.3.0 */
 #define GLUSTER_FOP_PROCCNT   GFS3_OP_MAXVALUE
 
+#define GLUSTER_FOP_VERSION_v2   400 /* 4.0.0 */
+
 /* Aggregator */
 #define GLUSTER_AGGREGATOR_PROGRAM 29852134 /* Completely random */
 #define GLUSTER_AGGREGATOR_VERSION 1
@@ -358,4 +371,5 @@ typedef enum gf_get_snapshot_info_type gf_get_snapshot_info_type;
 
 #define GD_VOLUME_NAME_MAX 256 /* Maximum size of volume name */
 
+#define GLUSTER_PROCESS_UUID_FMT "CTX_ID:%s-GRAPH_ID:%d-PID:%d-HOST:%s-PC_NAME:%s-RECON_NO:%s"
 #endif /* !_PROTOCOL_COMMON_H */
