@@ -54,6 +54,10 @@ af_inet_bind_to_port_lt_ceiling (struct rdma_cm_id *cm_id,
                                  struct sockaddr *sockaddr,
                                  socklen_t sockaddr_len, uint32_t ceiling)
 {
+#if GF_DISABLE_PRIVPORT_TRACKING
+        _assign_port (sockaddr, 0);
+        return rdma_bind_addr (cm_id, sockaddr);
+#else
         int32_t         ret                             = -1;
         uint16_t        port                            = ceiling - 1;
         unsigned char   ports[GF_PORT_ARRAY_SIZE]       = {0,};
@@ -100,6 +104,7 @@ loop:
         }
 
         return ret;
+#endif /* GF_DISABLE_PRIVPORT_TRACKING */
 }
 
 #if 0
@@ -444,7 +449,7 @@ gf_rdma_client_bind (rpc_transport_t *this, struct sockaddr *sockaddr,
         case AF_INET_SDP:
         case AF_INET:
                 *sockaddr_len = sizeof (struct sockaddr_in);
-
+        /* Fall through */
         case AF_INET6:
                 if (!this->bind_insecure) {
                         ret = af_inet_bind_to_port_lt_ceiling (cm_id, sockaddr,
@@ -508,7 +513,7 @@ gf_rdma_client_get_remote_sockaddr (rpc_transport_t *this,
         case AF_INET_SDP:
                 sockaddr->sa_family = AF_INET;
                 is_inet_sdp = 1;
-
+        /* Fall through */
         case AF_INET:
         case AF_INET6:
         case AF_UNSPEC:
@@ -581,7 +586,7 @@ gf_rdma_server_get_local_sockaddr (rpc_transport_t *this,
         case AF_INET_SDP:
                 is_inet_sdp = 1;
                 addr->sa_family = AF_INET;
-
+                /* Fall through */
         case AF_INET:
         case AF_INET6:
         case AF_UNSPEC:
@@ -680,7 +685,7 @@ gf_rdma_get_transport_identifiers (rpc_transport_t *this)
         case AF_INET_SDP:
                 is_inet_sdp = 1;
                 ((struct sockaddr *) &this->peerinfo.sockaddr)->sa_family = ((struct sockaddr *) &this->myinfo.sockaddr)->sa_family = AF_INET;
-
+                /* Fall through */
         case AF_INET:
         case AF_INET6: {
                 ret = fill_inet6_inet_identifiers (this,
