@@ -210,7 +210,7 @@ __afr_inode_write_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
                         local->transaction.unwind (frame, this);
                 }
 
-                local->transaction.resume (frame, this);
+                afr_transaction_resume (frame, this);
         }
 
         return 0;
@@ -317,10 +317,10 @@ afr_inode_write_fill (call_frame_t *frame, xlator_t *this, int child_index,
 		if (ret || !write_is_append)
 			local->append_write = _gf_false;
 
-		ret = dict_get_uint32 (xdata, GLUSTERFS_OPEN_FD_COUNT,
-				       &open_fd_count);
-		if (ret == -1)
-			goto unlock;
+                ret = dict_get_uint32 (xdata, GLUSTERFS_ACTIVE_FD_COUNT,
+                                       &open_fd_count);
+                if (ret < 0)
+                        goto unlock;
 		if (open_fd_count > local->open_fd_count) {
                         local->open_fd_count = open_fd_count;
                         local->update_open_fd_count = _gf_true;
@@ -344,14 +344,14 @@ afr_process_post_writev (call_frame_t *frame, xlator_t *this)
                    the xattrs are not reliably pointing at
                    a stale file.
                 */
-                afr_fd_report_unstable_write (this, local->fd);
+                afr_fd_report_unstable_write (this, local);
 
         __afr_inode_write_finalize (frame, this);
 
         afr_writev_handle_short_writes (frame, this);
 
         if (local->update_open_fd_count)
-                afr_handle_open_fd_count (frame, this);
+                local->inode_ctx->open_fd_count = local->open_fd_count;
 
 }
 
@@ -360,13 +360,10 @@ afr_writev_wind_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
                      int32_t op_ret, int32_t op_errno, struct iatt *prebuf,
                      struct iatt *postbuf, dict_t *xdata)
 {
-        afr_local_t     *local = NULL;
         call_frame_t    *fop_frame = NULL;
         int child_index = (long) cookie;
         int call_count  = -1;
 
-        local = frame->local;
-
         afr_inode_write_fill (frame, this, child_index, op_ret, op_errno,
                               prebuf, postbuf, xdata);
 
@@ -377,7 +374,7 @@ afr_writev_wind_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
 
                 if (!afr_txn_nothing_failed (frame, this)) {
                         //Don't unwind until post-op is complete
-                        local->transaction.resume (frame, this);
+                        afr_transaction_resume (frame, this);
                 } else {
                 /*
                  * Generally inode-write fops do transaction.unwind then
@@ -392,7 +389,7 @@ afr_writev_wind_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
 
                         fop_frame = afr_transaction_detach_fop_frame (frame);
                         afr_writev_copy_outvars (frame, fop_frame);
-                        local->transaction.resume (frame, this);
+                        afr_transaction_resume (frame, this);
                         afr_writev_unwind (fop_frame, this);
                 }
         }
@@ -465,8 +462,6 @@ afr_do_writev (call_frame_t *frame, xlator_t *this)
         local->op = GF_FOP_WRITE;
 
         local->transaction.wind   = afr_writev_wind;
-        local->transaction.fop    = __afr_txn_write_fop;
-        local->transaction.done   = __afr_txn_write_done;
         local->transaction.unwind = afr_transaction_writev_unwind;
 
         local->transaction.main_frame = frame;
@@ -537,10 +532,10 @@ afr_writev (call_frame_t *frame, xlator_t *this, fd_t *fd,
         if (ret)
                 goto out;
 
-	if (dict_set_uint32 (local->xdata_req, GLUSTERFS_OPEN_FD_COUNT, 4)) {
-		op_errno = ENOMEM;
-		goto out;
-	}
+        if (dict_set_uint32 (local->xdata_req, GLUSTERFS_ACTIVE_FD_COUNT, 4)) {
+                op_errno = ENOMEM;
+                goto out;
+        }
 
 	if (dict_set_uint32 (local->xdata_req, GLUSTERFS_WRITE_IS_APPEND, 4)) {
 		op_errno = ENOMEM;
@@ -655,8 +650,6 @@ afr_truncate (call_frame_t *frame, xlator_t *this,
 		goto out;
 
 	local->transaction.wind   = afr_truncate_wind;
-        local->transaction.fop    = __afr_txn_write_fop;
-        local->transaction.done   = __afr_txn_write_done;
         local->transaction.unwind = afr_truncate_unwind;
 
         loc_copy (&local->loc, loc);
@@ -783,8 +776,6 @@ afr_ftruncate (call_frame_t *frame, xlator_t *this, fd_t *fd, off_t offset,
         local->op = GF_FOP_FTRUNCATE;
 
 	local->transaction.wind   = afr_ftruncate_wind;
-        local->transaction.fop    = __afr_txn_write_fop;
-        local->transaction.done   = __afr_txn_write_done;
         local->transaction.unwind = afr_ftruncate_unwind;
 
         local->transaction.main_frame = frame;
@@ -891,8 +882,6 @@ afr_setattr (call_frame_t *frame, xlator_t *this, loc_t *loc, struct iatt *buf,
 		goto out;
 
         local->transaction.wind   = afr_setattr_wind;
-        local->transaction.fop    = __afr_txn_write_fop;
-        local->transaction.done   = __afr_txn_write_done;
         local->transaction.unwind = afr_setattr_unwind;
 
         loc_copy (&local->loc, loc);
@@ -998,8 +987,6 @@ afr_fsetattr (call_frame_t *frame, xlator_t *this,
 		goto out;
 
         local->transaction.wind   = afr_fsetattr_wind;
-        local->transaction.fop    = __afr_txn_write_fop;
-        local->transaction.done   = __afr_txn_write_done;
         local->transaction.unwind = afr_fsetattr_unwind;
 
         local->fd                 = fd_ref (fd);
@@ -1642,8 +1629,6 @@ afr_setxattr (call_frame_t *frame, xlator_t *this, loc_t *loc, dict_t *dict,
 		goto out;
 
         local->transaction.wind   = afr_setxattr_wind;
-        local->transaction.fop    = __afr_txn_write_fop;
-        local->transaction.done   = __afr_txn_write_done;
         local->transaction.unwind = afr_setxattr_unwind;
 
         loc_copy (&local->loc, loc);
@@ -1756,8 +1741,6 @@ afr_fsetxattr (call_frame_t *frame, xlator_t *this,
 		goto out;
 
         local->transaction.wind   = afr_fsetxattr_wind;
-        local->transaction.fop    = __afr_txn_write_fop;
-        local->transaction.done   = __afr_txn_write_done;
         local->transaction.unwind = afr_fsetxattr_unwind;
 
         local->fd                 = fd_ref (fd);
@@ -1871,8 +1854,6 @@ afr_removexattr (call_frame_t *frame, xlator_t *this,
 		goto out;
 
         local->transaction.wind   = afr_removexattr_wind;
-        local->transaction.fop    = __afr_txn_write_fop;
-        local->transaction.done   = __afr_txn_write_done;
         local->transaction.unwind = afr_removexattr_unwind;
 
         loc_copy (&local->loc, loc);
@@ -1980,8 +1961,6 @@ afr_fremovexattr (call_frame_t *frame, xlator_t *this, fd_t *fd,
 		goto out;
 
         local->transaction.wind   = afr_fremovexattr_wind;
-        local->transaction.fop    = __afr_txn_write_fop;
-        local->transaction.done   = __afr_txn_write_done;
         local->transaction.unwind = afr_fremovexattr_unwind;
 
         local->fd = fd_ref (fd);
@@ -2097,8 +2076,6 @@ afr_fallocate (call_frame_t *frame, xlator_t *this, fd_t *fd, int32_t mode,
         local->op = GF_FOP_FALLOCATE;
 
         local->transaction.wind   = afr_fallocate_wind;
-        local->transaction.fop    = __afr_txn_write_fop;
-        local->transaction.done   = __afr_txn_write_done;
         local->transaction.unwind = afr_fallocate_unwind;
 
         local->transaction.main_frame = frame;
@@ -2211,8 +2188,6 @@ afr_discard (call_frame_t *frame, xlator_t *this, fd_t *fd, off_t offset,
         local->op = GF_FOP_DISCARD;
 
         local->transaction.wind   = afr_discard_wind;
-        local->transaction.fop    = __afr_txn_write_fop;
-        local->transaction.done   = __afr_txn_write_done;
         local->transaction.unwind = afr_discard_unwind;
 
         local->transaction.main_frame = frame;
@@ -2322,8 +2297,6 @@ afr_zerofill (call_frame_t *frame, xlator_t *this, fd_t *fd, off_t offset,
         local->op = GF_FOP_ZEROFILL;
 
         local->transaction.wind   = afr_zerofill_wind;
-        local->transaction.fop    = __afr_txn_write_fop;
-        local->transaction.done   = __afr_txn_write_done;
         local->transaction.unwind = afr_zerofill_unwind;
 
         local->transaction.main_frame = frame;
@@ -2416,8 +2389,6 @@ afr_xattrop (call_frame_t *frame, xlator_t *this, loc_t *loc,
 		local->xdata_req = dict_ref (xdata);
 
         local->transaction.wind   = afr_xattrop_wind;
-        local->transaction.fop    = __afr_txn_write_fop;
-        local->transaction.done   = __afr_txn_write_done;
         local->transaction.unwind = afr_xattrop_unwind;
 
         loc_copy (&local->loc, loc);
@@ -2512,8 +2483,6 @@ afr_fxattrop (call_frame_t *frame, xlator_t *this, fd_t *fd,
 		local->xdata_req = dict_ref (xdata);
 
         local->transaction.wind   = afr_fxattrop_wind;
-        local->transaction.fop    = __afr_txn_write_fop;
-        local->transaction.done   = __afr_txn_write_done;
         local->transaction.unwind = afr_fxattrop_unwind;
 
 	local->fd = fd_ref (fd);
@@ -2542,3 +2511,109 @@ out:
 	AFR_STACK_UNWIND (fxattrop, frame, -1, op_errno, NULL, NULL);
         return 0;
 }
+
+
+int
+afr_fsync_unwind (call_frame_t *frame, xlator_t *this)
+{
+        afr_local_t *local = NULL;
+        call_frame_t   *main_frame = NULL;
+
+        local = frame->local;
+
+        main_frame = afr_transaction_detach_fop_frame (frame);
+        if (!main_frame)
+                return 0;
+
+        AFR_STACK_UNWIND (fsync, main_frame, local->op_ret, local->op_errno,
+                          &local->cont.inode_wfop.prebuf,
+                          &local->cont.inode_wfop.postbuf, local->xdata_rsp);
+
+        return 0;
+}
+
+
+int
+afr_fsync_wind_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
+                    int32_t op_ret, int32_t op_errno, struct iatt *prebuf,
+                    struct iatt *postbuf, dict_t *xdata)
+{
+        return __afr_inode_write_cbk (frame, cookie, this, op_ret, op_errno,
+                                      prebuf, postbuf, NULL, xdata);
+}
+
+
+int
+afr_fsync_wind (call_frame_t *frame, xlator_t *this, int subvol)
+{
+        afr_local_t *local = NULL;
+        afr_private_t *priv = NULL;
+
+        local = frame->local;
+        priv = this->private;
+
+        STACK_WIND_COOKIE (frame, afr_fsync_wind_cbk, (void *)(long) subvol,
+                           priv->children[subvol],
+                           priv->children[subvol]->fops->fsync,
+                           local->fd, local->cont.fsync.datasync,
+                           local->xdata_req);
+        return 0;
+}
+
+int
+afr_fsync (call_frame_t *frame, xlator_t *this, fd_t *fd, int32_t datasync,
+	   dict_t *xdata)
+{
+        afr_local_t *local = NULL;
+        call_frame_t *transaction_frame = NULL;
+        int ret = -1;
+        int32_t op_errno = ENOMEM;
+
+        transaction_frame = copy_frame (frame);
+        if (!transaction_frame)
+                goto out;
+
+	local = AFR_FRAME_INIT (transaction_frame, op_errno);
+	if (!local)
+		goto out;
+
+        if (xdata)
+                local->xdata_req = dict_copy_with_ref (xdata, NULL);
+        else
+                local->xdata_req = dict_new ();
+
+        if (!local->xdata_req)
+                goto out;
+
+        local->fd = fd_ref (fd);
+        ret = afr_set_inode_local (this, local, fd->inode);
+        if (ret)
+                goto out;
+
+        local->op = GF_FOP_FSYNC;
+        local->cont.fsync.datasync = datasync;
+
+	if (afr_fd_has_witnessed_unstable_write (this, fd->inode)) {
+		/* don't care. we only wanted to CLEAR the bit */
+	}
+
+        local->transaction.wind   = afr_fsync_wind;
+        local->transaction.unwind = afr_fsync_unwind;
+
+        local->transaction.main_frame = frame;
+
+        ret = afr_transaction (transaction_frame, this, AFR_DATA_TRANSACTION);
+        if (ret < 0) {
+                op_errno = -ret;
+                goto out;
+        }
+
+	return 0;
+out:
+	if (transaction_frame)
+		AFR_STACK_DESTROY (transaction_frame);
+
+	AFR_STACK_UNWIND (fsync, frame, -1, op_errno, NULL, NULL, NULL);
+
+        return 0;
+}
