@@ -66,7 +66,8 @@ GLFS_MSGID(AFR,
         AFR_MSG_NO_CHANGELOG,
         AFR_MSG_TIMER_CREATE_FAIL,
         AFR_MSG_SBRAIN_FAV_CHILD_POLICY,
-        AFR_MSG_INODE_CTX_GET_FAILED
+        AFR_MSG_INODE_CTX_GET_FAILED,
+        AFR_MSG_THIN_ARB
 );
 
 #endif /* !_AFR_MESSAGES_H_ */
