@@ -168,15 +168,6 @@ reconfigure (xlator_t *this, dict_t *options)
                           priv->data_self_heal_window_size, options,
                           uint32, out);
 
-        GF_OPTION_RECONF ("data-change-log", priv->data_change_log, options,
-                          bool, out);
-
-        GF_OPTION_RECONF ("metadata-change-log",
-                          priv->metadata_change_log, options, bool, out);
-
-        GF_OPTION_RECONF ("entry-change-log", priv->entry_change_log, options,
-                          bool, out);
-
         GF_OPTION_RECONF ("data-self-heal-algorithm",
                           priv->data_self_heal_algorithm, options, str, out);
 
@@ -245,9 +236,6 @@ reconfigure (xlator_t *this, dict_t *options)
         GF_OPTION_RECONF ("locking-scheme", priv->locking_scheme, options, str,
                           out);
         GF_OPTION_RECONF ("full-lock", priv->full_lock, options, bool, out);
-        GF_OPTION_RECONF ("use-compound-fops", priv->use_compound_fops,
-                          options, bool,
-                          out);
         GF_OPTION_RECONF ("granular-entry-heal", priv->esh_granular, options,
                           bool, out);
 
@@ -315,12 +303,20 @@ afr_pending_xattrs_init (afr_private_t *priv, xlator_t *this)
         char *ptr1 = NULL;
         char *xattrs_list = NULL;
         xlator_list_t *trav = NULL;
+        int child_count = -1;
 
         trav = this->children;
+        child_count = priv->child_count;
+        if (priv->thin_arbiter_count) {
+                /* priv->pending_key[THIN_ARBITER_BRICK_INDEX] is used as the
+                 * name of the thin arbiter file for persistance across add/
+                 * removal of DHT subvols.*/
+                child_count++;
+        }
 
         GF_OPTION_INIT ("afr-pending-xattr", xattrs_list, str, out);
         priv->pending_key = GF_CALLOC (sizeof (*priv->pending_key),
-                                       priv->child_count, gf_afr_mt_char);
+                                       child_count, gf_afr_mt_char);
         if (!priv->pending_key) {
                 ret = -ENOMEM;
                 goto out;
@@ -330,7 +326,7 @@ afr_pending_xattrs_init (afr_private_t *priv, xlator_t *this)
                         "Unable to fetch afr-pending-xattr option from volfile."
                         " Falling back to using client translator names. ");
 
-                while (i < priv->child_count) {
+                while (i < child_count) {
                         ret = gf_asprintf (&priv->pending_key[i], "%s.%s",
                                            AFR_XATTR_PREFIX,
                                            trav->xlator->name);
@@ -380,6 +376,7 @@ init (xlator_t *this)
         int            read_subvol_index = -1;
         char          *qtype       = NULL;
         char          *fav_child_policy = NULL;
+        char          *thin_arbiter = NULL;
 
         if (!this->children) {
                 gf_msg (this->name, GF_LOG_ERROR, 0,
@@ -409,6 +406,11 @@ init (xlator_t *this)
         priv->read_child = -1;
 
         GF_OPTION_INIT ("arbiter-count", priv->arbiter_count, uint32, out);
+        GF_OPTION_INIT ("thin-arbiter", thin_arbiter, str, out);
+        if (thin_arbiter && strlen(thin_arbiter) > 0) {
+                priv->thin_arbiter_count = 1;
+                priv->child_count--;
+        }
         INIT_LIST_HEAD (&priv->healing);
         INIT_LIST_HEAD (&priv->heal_waiting);
 
@@ -441,6 +443,9 @@ init (xlator_t *this)
         }
         GF_OPTION_INIT ("choose-local", priv->choose_local, bool, out);
 
+        priv->pending_reads = GF_CALLOC (sizeof(*priv->pending_reads),
+                                         priv->child_count, gf_afr_mt_atomic_t);
+
         GF_OPTION_INIT ("read-hash-mode", priv->hash_mode, uint32, out);
 
         priv->favorite_child = -1;
@@ -492,25 +497,12 @@ init (xlator_t *this)
 
         GF_OPTION_INIT ("iam-nfs-daemon", priv->nfsd.iamnfsd, bool, out);
 
-        GF_OPTION_INIT ("data-change-log", priv->data_change_log, bool, out);
-
-        GF_OPTION_INIT ("metadata-change-log", priv->metadata_change_log, bool,
-                        out);
-
-        GF_OPTION_INIT ("entry-change-log", priv->entry_change_log, bool, out);
-
         GF_OPTION_INIT ("optimistic-change-log", priv->optimistic_change_log,
                         bool, out);
 
-        GF_OPTION_INIT ("inodelk-trace", priv->inodelk_trace, bool, out);
-
-        GF_OPTION_INIT ("entrylk-trace", priv->entrylk_trace, bool, out);
-
         GF_OPTION_INIT ("pre-op-compat", priv->pre_op_compat, bool, out);
         GF_OPTION_INIT ("locking-scheme", priv->locking_scheme, str, out);
         GF_OPTION_INIT ("full-lock", priv->full_lock, bool, out);
-        GF_OPTION_INIT ("use-compound-fops", priv->use_compound_fops,
-                        bool, out);
         GF_OPTION_INIT ("granular-entry-heal", priv->esh_granular, bool, out);
 
         GF_OPTION_INIT ("eager-lock", priv->eager_lock, bool, out);
@@ -556,6 +548,10 @@ init (xlator_t *this)
                 ret = -ENOMEM;
                 goto out;
         }
+        /*Initialize to -ve ping timeout so that they are not considered
+         * in child-up events until ping-event comes*/
+        for (i = 0; i < child_count; i++)
+                priv->child_latency[i] = -1;
 
         priv->children = GF_CALLOC (sizeof (xlator_t *), child_count,
                                     gf_afr_mt_xlator_t);
@@ -636,13 +632,13 @@ struct xlator_fops fops = {
         .lk          = afr_lk,
         .flush       = afr_flush,
         .statfs      = afr_statfs,
-        .fsync       = afr_fsync,
         .fsyncdir    = afr_fsyncdir,
         .inodelk     = afr_inodelk,
         .finodelk    = afr_finodelk,
         .entrylk     = afr_entrylk,
         .fentrylk    = afr_fentrylk,
         .ipc         = afr_ipc,
+        .lease       = afr_lease,
 
         /* inode read */
         .access      = afr_access,
@@ -668,6 +664,7 @@ struct xlator_fops fops = {
         .zerofill    = afr_zerofill,
         .xattrop     = afr_xattrop,
         .fxattrop    = afr_fxattrop,
+        .fsync       = afr_fsync,
 
         /*inode open*/
         .opendir     = afr_opendir,
@@ -728,18 +725,19 @@ struct volume_options options[] = {
         { .key = {"read-hash-mode" },
           .type = GF_OPTION_TYPE_INT,
           .min = 0,
-          .max = 2,
+          .max = 3,
           .default_value = "1",
           .op_version = {2},
           .flags = OPT_FLAG_CLIENT_OPT | OPT_FLAG_SETTABLE | OPT_FLAG_DOC,
           .tags = {"replicate"},
           .description = "inode-read fops happen only on one of the bricks in "
                          "replicate. AFR will prefer the one computed using "
-                         "the method specified using this option"
-                         "0 = first up server, "
+                         "the method specified using this option.\n"
+                         "0 = first readable child of AFR, starting from 1st child.\n"
                          "1 = hash by GFID of file (all clients use "
-                                                    "same subvolume), "
-                         "2 = hash by GFID of file and client PID",
+                                                    "same subvolume).\n"
+                         "2 = hash by GFID of file and client PID.\n"
+                         "3 = brick having the least outstanding read requests."
         },
         { .key  = {"choose-local" },
           .type = GF_OPTION_TYPE_BOOL,
@@ -794,7 +792,7 @@ struct volume_options options[] = {
         { .key   = {"halo-max-latency"},
           .type  = GF_OPTION_TYPE_INT,
           .min   = 1,
-          .max   = 99999,
+          .max   = AFR_HALO_MAX_LATENCY,
           .default_value = "5",
           .op_version = {GD_OP_VERSION_3_11_0},
           .flags = OPT_FLAG_CLIENT_OPT | OPT_FLAG_SETTABLE | OPT_FLAG_DOC,
@@ -906,9 +904,8 @@ struct volume_options options[] = {
           .op_version = {1},
           .flags = OPT_FLAG_CLIENT_OPT | OPT_FLAG_SETTABLE | OPT_FLAG_DOC,
           .tags = {"replicate"},
-          .description = "Data fops like write/truncate will not perform "
-                         "pre/post fop changelog operations in afr transaction "
-                         "if this option is disabled"
+          .description = "This option exists only for backward compatibility "
+                         "and configuring it doesn't have any effect"
         },
         { .key  = {"metadata-change-log"},
           .type = GF_OPTION_TYPE_BOOL,
@@ -916,9 +913,8 @@ struct volume_options options[] = {
           .op_version = {1},
           .flags = OPT_FLAG_CLIENT_OPT | OPT_FLAG_SETTABLE | OPT_FLAG_DOC,
           .tags = {"replicate"},
-          .description = "Metadata fops like setattr/setxattr will not perform "
-                         "pre/post fop changelog operations in afr transaction "
-                         "if this option is disabled"
+          .description = "This option exists only for backward compatibility "
+                         "and configuring it doesn't have any effect"
         },
         { .key  = {"entry-change-log"},
           .type = GF_OPTION_TYPE_BOOL,
@@ -926,9 +922,8 @@ struct volume_options options[] = {
           .op_version = {1},
           .flags = OPT_FLAG_CLIENT_OPT | OPT_FLAG_SETTABLE | OPT_FLAG_DOC,
           .tags = {"replicate"},
-          .description = "Entry fops like create/unlink will not perform "
-                         "pre/post fop changelog operations in afr transaction "
-                         "if this option is disabled"
+          .description = "This option exists only for backward compatibility "
+                         "and configuring it doesn't have any effect"
         },
         { .key  = {"optimistic-change-log"},
           .type = GF_OPTION_TYPE_BOOL,
@@ -1097,7 +1092,7 @@ struct volume_options options[] = {
 	},
         { .key  = {"heal-timeout"},
           .type = GF_OPTION_TYPE_INT,
-          .min  = 60,
+          .min  = 5,
           .max  = INT_MAX,
           .default_value = "600",
           .op_version = {2},
@@ -1123,6 +1118,13 @@ struct volume_options options[] = {
           .type = GF_OPTION_TYPE_INT,
           .description = "subset of child_count. Has to be 0 or 1."
         },
+        { .key = {"thin-arbiter"},
+          .type = GF_OPTION_TYPE_STR,
+          .op_version = {GD_OP_VERSION_4_1_0},
+          .flags = OPT_FLAG_SETTABLE,
+          .tags = {"replicate"},
+          .description = "contains host:path of thin abriter brick",
+        },
         { .key   = {"shd-max-threads"},
           .type  = GF_OPTION_TYPE_INT,
           .min   = 1,
@@ -1205,9 +1207,8 @@ struct volume_options options[] = {
           .op_version = {GD_OP_VERSION_3_8_4},
           .flags = OPT_FLAG_CLIENT_OPT | OPT_FLAG_SETTABLE | OPT_FLAG_DOC,
           .tags = {"replicate"},
-          .description = "Use compound fops framework to modify afr "
-                         "transaction such that network roundtrips are "
-                         "reduced, thus improving the performance.",
+          .description = "This option exists only for backward compatibility "
+                         "and configuring it doesn't have any effect"
         },
         { .key  = {NULL} },
 };
