@@ -35,7 +35,10 @@
 #define AFR_DEFAULT_SPB_CHOICE_TIMEOUT 300 /*in seconds*/
 
 #define ARBITER_BRICK_INDEX 2
+#define THIN_ARBITER_BRICK_INDEX 2
+#define THIN_ARBITER_DOM1 "afr.ta.domain-1"
 
+#define AFR_HALO_MAX_LATENCY 99999
 typedef int (*afr_lock_cbk_t) (call_frame_t *frame, xlator_t *this);
 
 typedef int (*afr_read_txn_wind_t) (call_frame_t *frame, xlator_t *this, int subvol);
@@ -44,10 +47,6 @@ typedef int (*afr_inode_refresh_cbk_t) (call_frame_t *frame, xlator_t *this, int
 
 typedef int (*afr_changelog_resume_t) (call_frame_t *frame, xlator_t *this);
 
-typedef int (*afr_compound_cbk_t) (call_frame_t *frame, void *cookie,
-                                   xlator_t *this, int op_ret, int op_errno,
-                                   void *data, dict_t *xdata);
-
 #define AFR_COUNT(array,max) ({int __i; int __res = 0; for (__i = 0; __i < max; __i++) if (array[__i]) __res++; __res;})
 #define AFR_INTERSECT(dst,src1,src2,max) ({int __i; for (__i = 0; __i < max; __i++) dst[__i] = src1[__i] && src2[__i];})
 #define AFR_CMP(a1,a2,len) ({int __cmp = 0; int __i; for (__i = 0; __i < len; __i++) if (a1[__i] != a2[__i]) { __cmp = 1; break;} __cmp;})
@@ -84,10 +83,12 @@ typedef struct _afr_private {
         unsigned int child_count;     /* total number of children   */
         unsigned int arbiter_count;   /*subset of child_count.
                                         Has to be 0 or 1.*/
+        unsigned int thin_arbiter_count; /* 0 or 1 at the moment.*/
 
         xlator_t **children;
 
         inode_t *root_inode;
+        uuid_t ta_gfid; /*For thin arbiter.*/
 
         unsigned char *child_up;
         int64_t *child_latency;
@@ -114,22 +115,16 @@ typedef struct _afr_private {
         gf_boolean_t metadata_self_heal;   /* on/off */
         gf_boolean_t entry_self_heal;      /* on/off */
 
-        gf_boolean_t data_change_log;       /* on/off */
-        gf_boolean_t metadata_change_log;   /* on/off */
-        gf_boolean_t entry_change_log;      /* on/off */
-
 	gf_boolean_t metadata_splitbrain_forced_heal; /* on/off */
         int read_child;               /* read-subvolume */
         unsigned int hash_mode;       /* for when read_child is not set */
+        gf_atomic_t *pending_reads; /*No. of pending read cbks per child.*/
         int favorite_child;  /* subvolume to be preferred in resolving
                                          split-brain cases */
 
         afr_favorite_child_policy fav_child_policy;/*Policy to use for automatic
                                                  resolution of split-brains.*/
 
-        gf_boolean_t inodelk_trace;
-        gf_boolean_t entrylk_trace;
-
         unsigned int wait_count;      /* # of servers to wait for success */
 
         gf_timer_t *timer;      /* launched when parent up is received */
@@ -180,7 +175,6 @@ typedef struct _afr_private {
         gf_boolean_t           full_lock;
         gf_boolean_t           esh_granular;
         gf_boolean_t           consistent_io;
-        gf_boolean_t           use_compound_fops;
 } afr_private_t;
 
 
@@ -191,33 +185,6 @@ typedef enum {
         AFR_ENTRY_RENAME_TRANSACTION,  /* rename */
 } afr_transaction_type;
 
-typedef enum {
-        AFR_TRANSACTION_LK,
-        AFR_SELFHEAL_LK,
-} transaction_lk_type_t;
-
-typedef enum {
-        AFR_LOCK_OP,
-        AFR_UNLOCK_OP,
-} afr_lock_op_type_t;
-
-typedef enum {
-        AFR_DATA_SELF_HEAL_LK,
-        AFR_METADATA_SELF_HEAL_LK,
-        AFR_ENTRY_SELF_HEAL_LK,
-}selfheal_lk_type_t;
-
-typedef enum {
-        AFR_INODELK_TRANSACTION,
-        AFR_INODELK_NB_TRANSACTION,
-        AFR_ENTRYLK_TRANSACTION,
-        AFR_ENTRYLK_NB_TRANSACTION,
-        AFR_INODELK_SELFHEAL,
-        AFR_INODELK_NB_SELFHEAL,
-        AFR_ENTRYLK_SELFHEAL,
-        AFR_ENTRYLK_NB_SELFHEAL,
-} afr_lock_call_type_t;
-
 /*
   xattr format: trusted.afr.volume = [x y z]
   x - data pending
@@ -267,20 +234,13 @@ typedef struct {
 int
 afr_entry_lockee_cmp (const void *l1, const void *l2);
 
-typedef struct {
-        char    *domain; /* Domain on which inodelk is taken */
-        struct gf_flock flock;
-        unsigned char *locked_nodes;
-        int32_t lock_count;
-} afr_inodelk_t;
-
 typedef struct {
         loc_t *lk_loc;
 
         int                     lockee_count;
         afr_entry_lockee_t      lockee[AFR_LOCKEE_COUNT_MAX];
 
-        afr_inodelk_t       inodelk[AFR_DOM_COUNT_MAX];
+        struct gf_flock flock;
         const char *lk_basename;
         const char *lower_basename;
         const char *higher_basename;
@@ -290,13 +250,9 @@ typedef struct {
         unsigned char *locked_nodes;
         unsigned char *lower_locked_nodes;
 
-        selfheal_lk_type_t selfheal_lk_type;
-        transaction_lk_type_t transaction_lk_type;
-
         int32_t lock_count;
         int32_t entrylk_lock_count;
 
-        uint64_t lock_number;
         int32_t lk_call_count;
         int32_t lk_expected_count;
         int32_t lk_attempted_count;
@@ -334,37 +290,9 @@ typedef enum {
 } afr_fd_open_status_t;
 
 typedef struct {
-        unsigned int *pre_op_done[AFR_NUM_CHANGE_LOGS];
-	int inherited[AFR_NUM_CHANGE_LOGS];
-	int on_disk[AFR_NUM_CHANGE_LOGS];
         afr_fd_open_status_t *opened_on; /* which subvolumes the fd is open on */
-
-        unsigned int *lock_piggyback;
-        unsigned int *lock_acquired;
-
         int flags;
 
-	/* used for delayed-post-op optimization */
-	pthread_mutex_t    delay_lock;
-	gf_timer_t        *delay_timer;
-	call_frame_t      *delay_frame;
-
-	/* set if any write on this fd was a non stable write
-	   (i.e, without O_SYNC or O_DSYNC)
-	*/
-	gf_boolean_t      witnessed_unstable_write;
-
-	/* @open_fd_count:
-	   Number of open FDs queried from the server, as queried through
-	   xdata in FOPs. Currently, used to decide if eager-locking must be
-	   temporarily disabled.
-	*/
-        uint32_t        open_fd_count;
-
-
-	/* list of frames currently in progress */
-	struct list_head  eager_locked;
-
 	/* the subvolume on which the latest sequence of readdirs (starting
 	   at offset 0) has begun. Till the next readdir request with 0 offset
 	   arrives, we continue to read off this subvol.
@@ -378,6 +306,20 @@ typedef enum {
         AFR_FOP_LOCK_QUORUM_FAILED,
 } afr_fop_lock_state_t;
 
+typedef struct _afr_inode_lock_t {
+        unsigned int event_generation;
+        gf_boolean_t    release;
+        gf_boolean_t    acquired;
+        gf_timer_t        *delay_timer;
+        struct list_head  owners; /*Transactions that are performing fop*/
+        struct list_head  post_op;/*Transactions that are done with the fop
+                                   *So can not conflict with the fops*/
+        struct list_head waiting;/*Transaction that are waiting for
+                                   *conflicting transactions to complete*/
+        struct list_head frozen;/*Transactions that need to go as part of
+                                 * next batch of eager-lock*/
+} afr_lock_t;
+
 typedef struct _afr_inode_ctx {
         uint64_t        read_subvol;
         uint64_t        write_subvol;
@@ -385,6 +327,23 @@ typedef struct _afr_inode_ctx {
         int             spb_choice;
         gf_timer_t      *timer;
         gf_boolean_t    need_refresh;
+        unsigned int *pre_op_done[AFR_NUM_CHANGE_LOGS];
+        int inherited[AFR_NUM_CHANGE_LOGS];
+        int on_disk[AFR_NUM_CHANGE_LOGS];
+
+        /* set if any write on this fd was a non stable write
+           (i.e, without O_SYNC or O_DSYNC)
+        */
+        gf_boolean_t      witnessed_unstable_write;
+
+        /* @open_fd_count:
+           Number of open FDs queried from the server, as queried through
+           xdata in FOPs. Currently, used to decide if eager-locking must be
+           temporarily disabled.
+        */
+        uint32_t        open_fd_count;
+        /*Only 2 types of transactions support eager-locks now. DATA/METADATA*/
+        afr_lock_t lock[2];
 } afr_inode_ctx_t;
 
 
@@ -472,6 +431,8 @@ typedef struct _afr_local {
 	unsigned char *readable;
 	unsigned char *readable2; /*For rename transaction*/
 
+        int read_subvol; /* Current read subvolume */
+
 	afr_inode_refresh_cbk_t refreshfn;
 
 	/* @refreshinode:
@@ -499,7 +460,6 @@ typedef struct _afr_local {
         dict_t  *dict;
 
         int      optimistic_change_log;
-	gf_boolean_t      delayed_post_op;
 
 	/* Is the current writev() going to perform a stable write?
 	   i.e, is fd->flags or @flags writev param have O_SYNC or
@@ -725,13 +685,24 @@ typedef struct _afr_local {
                         gf_seek_what_t what;
                 } seek;
 
+                struct {
+                        int32_t datasync;
+                } fsync;
+
+                struct {
+                        struct gf_lease user_lease;
+                        struct gf_lease ret_lease;
+                        unsigned char *locked_nodes;
+                } lease;
+
+
         } cont;
 
         struct {
                 off_t start, len;
 
                 gf_boolean_t    eager_lock_on;
-                int *eager_lock;
+                gf_boolean_t    do_eager_unlock;
 
                 char *basename;
                 char *new_basename;
@@ -745,7 +716,8 @@ typedef struct _afr_local {
 		   of the transaction frame */
 		call_stub_t      *resume_stub;
 
-		struct list_head  eager_locked;
+		struct list_head  owner_list;
+                struct list_head  wait_list;
 
                 unsigned char   *pre_op;
 
@@ -806,16 +778,11 @@ typedef struct _afr_local {
 		*/
 		afr_changelog_resume_t changelog_resume;
 
-                call_frame_t *main_frame;
+                call_frame_t *main_frame; /*Fop frame*/
+                call_frame_t *frame; /*Transaction frame*/
 
                 int (*wind) (call_frame_t *frame, xlator_t *this, int subvol);
 
-                int (*fop) (call_frame_t *frame, xlator_t *this);
-
-                int (*done) (call_frame_t *frame, xlator_t *this);
-
-                int (*resume) (call_frame_t *frame, xlator_t *this);
-
                 int (*unwind) (call_frame_t *frame, xlator_t *this);
 
                 /* post-op hook */
@@ -839,9 +806,7 @@ typedef struct _afr_local {
         call_frame_t *heal_frame;
 
         gf_boolean_t need_full_crawl;
-        gf_boolean_t compound;
         afr_fop_lock_state_t fop_lock_state;
-        compound_args_t *c_args;
 
         gf_boolean_t is_read_txn;
         afr_inode_ctx_t *inode_ctx;
@@ -975,10 +940,6 @@ afr_blocking_lock (call_frame_t *frame, xlator_t *this);
 int
 afr_internal_lock_finish (call_frame_t *frame, xlator_t *this);
 
-int
-afr_lk_transfer_datalock (call_frame_t *dst, call_frame_t *src, char *dom,
-                          unsigned int child_count);
-
 int
 __afr_fd_ctx_set (xlator_t *this, fd_t *fd);
 
@@ -1028,6 +989,8 @@ afr_cleanup_fd_ctx (xlator_t *this, fd_t *fd);
                         __this = frame->this;                   \
                         afr_handle_inconsistent_fop (frame, &__op_ret,\
                                                      &__op_errno);\
+                        if (__local && __local->is_read_txn) \
+                                afr_pending_read_decrement (__this->private, __local->read_subvol); \
                         frame->local = NULL;                    \
                 }                                               \
                                                                 \
@@ -1059,7 +1022,7 @@ afr_cleanup_fd_ctx (xlator_t *this, fd_t *fd);
 		afr_local_cleanup (frame->local, THIS);		       \
 		mem_put (frame->local);				       \
 		frame->local = NULL; };				       \
-	frame->local;})
+	frame->local; })
 
 #define AFR_STACK_RESET(frame)                                         \
         do {                                                           \
@@ -1103,8 +1066,7 @@ int
 afr_local_init (afr_local_t *local, afr_private_t *priv, int32_t *op_errno);
 
 int
-afr_internal_lock_init (afr_internal_lock_t *lk, size_t child_count,
-                        transaction_lk_type_t lk_type);
+afr_internal_lock_init (afr_internal_lock_t *lk, size_t child_count);
 
 int
 afr_higher_errno (int32_t old_errno, int32_t new_errno);
@@ -1147,22 +1109,10 @@ afr_filter_xattrs (dict_t *xattr);
 #define AFR_QUORUM_AUTO INT_MAX
 
 int
-afr_fd_report_unstable_write (xlator_t *this, fd_t *fd);
+afr_fd_report_unstable_write (xlator_t *this, afr_local_t *local);
 
 gf_boolean_t
-afr_fd_has_witnessed_unstable_write (xlator_t *this, fd_t *fd);
-
-void
-afr_delayed_changelog_wake_resume (xlator_t *this, fd_t *fd, call_stub_t *stub);
-
-int
-afr_inodelk_init (afr_inodelk_t *lk, char *dom, size_t child_count);
-
-void
-afr_handle_open_fd_count (call_frame_t *frame, xlator_t *this);
-
-void
-afr_remove_eager_lock_stub (afr_local_t *local);
+afr_fd_has_witnessed_unstable_write (xlator_t *this, inode_t *inode);
 
 void
 afr_reply_wipe (struct afr_reply *reply);
@@ -1248,14 +1198,8 @@ afr_writev_copy_outvars (call_frame_t *src_frame, call_frame_t *dst_frame);
 void
 afr_update_uninodelk (afr_local_t *local, afr_internal_lock_t *int_lock,
                     int32_t child_index);
-gf_boolean_t
-afr_can_compound_pre_op_and_op (afr_private_t *priv, glusterfs_fop_t fop);
-
-afr_compound_cbk_t
-afr_pack_fop_args (call_frame_t *frame, compound_args_t *args,
-                   glusterfs_fop_t fop, int index);
 int
-afr_is_inodelk_transaction(afr_local_t *local);
+afr_is_inodelk_transaction(afr_transaction_type type);
 
 afr_fd_ctx_t *
 __afr_fd_ctx_get (fd_t *fd, xlator_t *this);
@@ -1274,6 +1218,9 @@ afr_is_symmetric_error (call_frame_t *frame, xlator_t *this);
 int
 __afr_inode_ctx_get (xlator_t *this, inode_t *inode, afr_inode_ctx_t **ctx);
 
+uint64_t
+afr_write_subvol_get (call_frame_t *frame, xlator_t *this);
+
 int
 afr_write_subvol_set (call_frame_t *frame, xlator_t *this);
 
