@@ -18,6 +18,7 @@
 #include "defaults.h"
 
 #include <sys/time.h>
+#include "events.h"
 
 
 int
@@ -38,14 +39,14 @@ dht_du_info_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
 	conf = this->private;
 	prev = cookie;
 
-	if (op_ret == -1) {
+	if (op_ret == -1 || !statvfs) {
 		gf_msg (this->name, GF_LOG_WARNING, op_errno,
                         DHT_MSG_GET_DISK_INFO_ERROR,
 			"failed to get disk info from %s", prev->name);
 		goto out;
 	}
 
-	if (statvfs && statvfs->f_blocks) {
+	if (statvfs->f_blocks) {
 		percent = (statvfs->f_bavail * 100) / statvfs->f_blocks;
 		bytes = (statvfs->f_bavail * statvfs->f_frsize);
                 /*
@@ -60,7 +61,7 @@ dht_du_info_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
                 chunks = (statvfs->f_blocks + bpc - 1) / bpc;
 	}
 
-	if (statvfs && statvfs->f_files) {
+	if (statvfs->f_files) {
 		percent_inodes = (statvfs->f_ffree * 100) / statvfs->f_files;
 	} else {
                 /*
@@ -81,7 +82,11 @@ dht_du_info_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
 				conf->du_stats[i].avail_space   = bytes;
 				conf->du_stats[i].avail_inodes  = percent_inodes;
                                 conf->du_stats[i].chunks        = chunks;
-				gf_msg_debug (this->name, 0,
+                                conf->du_stats[i].total_blocks  = statvfs->f_blocks;
+                                conf->du_stats[i].avail_blocks  = statvfs->f_bavail;
+                                conf->du_stats[i].frsize        = statvfs->f_frsize;
+
+                                gf_msg_debug (this->name, 0,
 				              "subvolume '%s': avail_percent "
 					      "is: %.2f and avail_space "
                                               "is: %" PRIu64" and avail_inodes"
@@ -217,10 +222,12 @@ gf_boolean_t
 dht_is_subvol_filled (xlator_t *this, xlator_t *subvol)
 {
 	int         i = 0;
+        char      vol_name[256];
 	dht_conf_t *conf = NULL;
 	gf_boolean_t subvol_filled_inodes = _gf_false;
 	gf_boolean_t subvol_filled_space = _gf_false;
 	gf_boolean_t is_subvol_filled = _gf_false;
+        double usage = 0;
 
 	conf = this->private;
 
@@ -255,23 +262,38 @@ dht_is_subvol_filled (xlator_t *this, xlator_t *subvol)
 
 	if (subvol_filled_space && conf->subvolume_status[i]) {
 		if (!(conf->du_stats[i].log++ % (GF_UNIVERSAL_ANSWER * 10))) {
+                        usage = 100 - conf->du_stats[i].avail_percent;
+
 			gf_msg (this->name, GF_LOG_WARNING, 0,
                                 DHT_MSG_SUBVOL_INSUFF_SPACE,
 				"disk space on subvolume '%s' is getting "
 				"full (%.2f %%), consider adding more bricks",
-				subvol->name,
-				(100 - conf->du_stats[i].avail_percent));
+				subvol->name, usage);
+
+                        strncpy(vol_name, this->name, sizeof(vol_name));
+                        vol_name[(strlen(this->name)-4)] = '\0';
+
+                        gf_event(EVENT_DHT_DISK_USAGE,
+                                 "volume=%s;subvol=%s;usage=%.2f %%",
+                                 vol_name, subvol->name, usage);
 		}
 	}
 
 	if (subvol_filled_inodes && conf->subvolume_status[i]) {
 		if (!(conf->du_stats[i].log++ % (GF_UNIVERSAL_ANSWER * 10))) {
+                        usage = 100 - conf->du_stats[i].avail_inodes;
 			gf_msg (this->name, GF_LOG_CRITICAL, 0,
                                 DHT_MSG_SUBVOL_INSUFF_INODES,
 				"inodes on subvolume '%s' are at "
 				"(%.2f %%), consider adding more bricks",
-				subvol->name,
-				(100 - conf->du_stats[i].avail_inodes));
+				subvol->name, usage);
+
+                        strncpy(vol_name, this->name, sizeof(vol_name));
+                        vol_name[(strlen(this->name)-4)] = '\0';
+
+                        gf_event(EVENT_DHT_INODES_USAGE,
+                                 "volume=%s;subvol=%s;usage=%.2f %%",
+                                  vol_name, subvol->name, usage);
 		}
 	}
 
@@ -311,8 +333,8 @@ dht_free_disk_available_subvol (xlator_t *this, xlator_t *subvol,
 
         LOCK (&conf->subvolume_lock);
 	{
-                avail_subvol = dht_subvol_with_free_space_inodes(this, subvol,
-                                                                 layout);
+                avail_subvol = dht_subvol_with_free_space_inodes(this, subvol, NULL,
+                                                                 layout, 0);
                 if(!avail_subvol)
                 {
                         avail_subvol = dht_subvol_maxspace_nonzeroinode(this,
@@ -336,8 +358,8 @@ out:
 }
 
 static inline
-int32_t dht_subvol_has_err (dht_conf_t *conf, xlator_t *this,
-                                         dht_layout_t *layout)
+int32_t dht_subvol_has_err (dht_conf_t *conf, xlator_t *this, xlator_t *ignore,
+                            dht_layout_t *layout)
 {
         int ret = -1;
         int i   = 0;
@@ -345,6 +367,13 @@ int32_t dht_subvol_has_err (dht_conf_t *conf, xlator_t *this,
         if (!this || !layout)
                 goto out;
 
+        /* this check is meant for rebalance process. The source of the file
+         * should be ignored for space check */
+        if (this == ignore) {
+                goto out;
+        }
+
+
         /* check if subvol has layout errors, before selecting it */
         for (i = 0; i < layout->cnt; i++) {
                 if (!strcmp (layout->list[i].xlator->name, this->name) &&
@@ -372,13 +401,18 @@ out:
 
 /*Get subvolume which has both space and inodes more than the min criteria*/
 xlator_t *
-dht_subvol_with_free_space_inodes(xlator_t *this, xlator_t *subvol,
-                                  dht_layout_t *layout)
+dht_subvol_with_free_space_inodes(xlator_t *this, xlator_t *subvol, xlator_t *ignore,
+                                  dht_layout_t *layout, uint64_t filesize)
 {
         int i = 0;
         double max = 0;
         double max_inodes = 0;
         int    ignore_subvol = 0;
+        uint64_t total_blocks = 0;
+        uint64_t avail_blocks = 0;
+        uint64_t frsize = 0;
+        double   post_availspace = 0;
+        double   post_percent = 0;
 
         xlator_t *avail_subvol = NULL;
         dht_conf_t *conf = NULL;
@@ -389,7 +423,7 @@ dht_subvol_with_free_space_inodes(xlator_t *this, xlator_t *subvol,
                 /* check if subvol has layout errors and also it is not a
                  * decommissioned brick, before selecting it */
                 ignore_subvol = dht_subvol_has_err (conf, conf->subvolumes[i],
-                                                    layout);
+                                                    ignore, layout);
                 if (ignore_subvol)
                         continue;
 
@@ -401,6 +435,9 @@ dht_subvol_with_free_space_inodes(xlator_t *this, xlator_t *subvol,
                                 max = conf->du_stats[i].avail_percent;
                                 max_inodes = conf->du_stats[i].avail_inodes;
                                 avail_subvol = conf->subvolumes[i];
+                                total_blocks = conf->du_stats[i].total_blocks;
+                                avail_blocks = conf->du_stats[i].avail_blocks;
+                                frsize       = conf->du_stats[i].frsize;
                         }
                 }
 
@@ -416,6 +453,19 @@ dht_subvol_with_free_space_inodes(xlator_t *this, xlator_t *subvol,
                 }
         }
 
+        if (avail_subvol) {
+                if (conf->disk_unit == 'p') {
+                        post_availspace = (avail_blocks * frsize) - filesize;
+                        post_percent = (post_availspace * 100) / (total_blocks * frsize);
+                        if (post_percent < conf->min_free_disk)
+                                avail_subvol = NULL;
+                }
+                if (conf->disk_unit != 'p') {
+                        if ((max - filesize) < conf->min_free_disk)
+                                avail_subvol = NULL;
+                }
+        }
+
         return avail_subvol;
 }
 
@@ -438,7 +488,7 @@ dht_subvol_maxspace_nonzeroinode (xlator_t *this, xlator_t *subvol,
                 /* check if subvol has layout errors and also it is not a
                  * decommissioned brick, before selecting it*/
 
-                ignore_subvol = dht_subvol_has_err (conf, conf->subvolumes[i],
+                ignore_subvol = dht_subvol_has_err (conf, conf->subvolumes[i], NULL,
                                                     layout);
                 if (ignore_subvol)
                         continue;
