@@ -78,7 +78,7 @@ dht_open2 (xlator_t *this, xlator_t *subvol, call_frame_t *frame, int ret)
                 goto out;
 
         local = frame->local;
-        op_errno = ENOENT;
+        op_errno = local->op_errno;
 
         if (we_are_not_migrating (ret)) {
                 /* This DHT layer is not migrating the file */
@@ -400,8 +400,8 @@ dht_fstat (call_frame_t *frame, xlator_t *this, fd_t *fd, dict_t *xdata)
 
         layout = local->layout;
         if (!layout) {
-                gf_msg_debug (this->name, 0,
-                              "no layout for fd=%p", fd);
+                gf_msg (this->name, GF_LOG_ERROR, 0, 0,
+                        "no layout for fd=%p", fd);
                 op_errno = EINVAL;
                 goto err;
         }
