@@ -11,14 +11,23 @@
 #ifndef __EC_H__
 #define __EC_H__
 
+#include "ec-method.h"
+
 #define EC_XATTR_PREFIX  "trusted.ec."
 #define EC_XATTR_CONFIG  EC_XATTR_PREFIX"config"
 #define EC_XATTR_SIZE    EC_XATTR_PREFIX"size"
 #define EC_XATTR_VERSION EC_XATTR_PREFIX"version"
 #define EC_XATTR_HEAL    EC_XATTR_PREFIX"heal"
 #define EC_XATTR_DIRTY   EC_XATTR_PREFIX"dirty"
-
+#define EC_STRIPE_CACHE_MAX_SIZE    10
 #define EC_VERSION_SIZE 2
 #define EC_SHD_INODE_LRU_LIMIT          10
 
+#define EC_MAX_FRAGMENTS EC_METHOD_MAX_FRAGMENTS
+/* The maximum number of nodes is derived from the maximum allowed fragments
+ * using the rule that redundancy cannot be equal or greater than the number
+ * of fragments.
+ */
+#define EC_MAX_NODES min(EC_MAX_FRAGMENTS * 2 - 1, EC_METHOD_MAX_NODES)
+
 #endif /* __EC_H__ */
