@@ -54,7 +54,9 @@ GLFS_MSGID(BITROT_STUB,
         BRS_MSG_BAD_OBJ_THREAD_FAIL,
         BRS_MSG_BAD_OBJ_DIR_CLOSE_FAIL,
         BRS_MSG_LINK_FAIL,
-        BRS_MSG_BAD_OBJ_UNLINK_FAIL
+        BRS_MSG_BAD_OBJ_UNLINK_FAIL,
+        BRS_MSG_DICT_SET_FAILED,
+        BRS_MSG_PATH_GET_FAILED
 );
 
 #endif /* !_BITROT_STUB_MESSAGES_H_ */
