@@ -8,11 +8,11 @@
    cases as published by the Free Software Foundation.
 */
 
+
 #ifndef _GLUSTERD_PEER_UTILS_H
 #define _GLUSTERD_PEER_UTILS_H
 
 #include "glusterd.h"
-#include "glusterd-utils.h"
 
 int32_t
 glusterd_peerinfo_cleanup (glusterd_peerinfo_t *peerinfo);
@@ -87,4 +87,7 @@ gd_add_peer_detail_to_dict (glusterd_peerinfo_t *peerinfo, dict_t *friends,
                             int count);
 glusterd_peerinfo_t *
 glusterd_peerinfo_find_by_generation (uint32_t generation);
+
+int
+glusterd_get_peers_count ();
 #endif /* _GLUSTERD_PEER_UTILS_H */
