@@ -22,7 +22,6 @@
 #include "xlator.h"
 #include "logging.h"
 #include "call-stub.h"
-#include "fd.h"
 #include "byte-order.h"
 #include "glusterd-mem-types.h"
 #include "rpcsvc.h"
@@ -198,6 +197,7 @@ typedef struct {
         int32_t                    workers;
         uint32_t                   blockers;
         uint32_t                   mgmt_v3_lock_timeout;
+        gf_boolean_t               restart_bricks;
 } glusterd_conf_t;
 
 
@@ -1082,7 +1082,7 @@ glusterd_add_volume_detail_to_dict (glusterd_volinfo_t *volinfo,
                                     dict_t  *volumes, int   count);
 
 int
-glusterd_restart_bricks (glusterd_conf_t *conf);
+glusterd_restart_bricks ();
 
 int32_t
 glusterd_volume_txn (rpcsvc_request_t *req, char *volname, int flags,
