@@ -8,11 +8,6 @@
  * cases as published by the Free Software Foundation.
  */
 
-#ifndef _CONFIG_H
-#define _CONFIG_H
-#include "config.h"
-#endif
-
 #include "defaults.h"
 #include "rpcsvc.h"
 #include "dict.h"
@@ -21,7 +16,6 @@
 #include "mem-pool.h"
 #include "logging.h"
 #include "nfs-fops.h"
-#include "inode.h"
 #include "nfs3.h"
 #include "nfs-mem-types.h"
 #include "nfs3-helpers.h"
