@@ -1121,10 +1121,6 @@ is_client_rpc_init_command (dict_t *dict, xlator_t *this,
         gf_boolean_t ret      = _gf_false;
         int          dict_ret = -1;
 
-        if (!strstr (this->name, "replace-brick")) {
-                gf_msg_trace (this->name, 0, "name is !replace-brick");
-                goto out;
-        }
         dict_ret = dict_get_str (dict, CLIENT_CMD_CONNECT, value);
         if (dict_ret) {
                 gf_msg_trace (this->name, 0, "key %s not present",
@@ -1165,7 +1161,7 @@ out:
 
 }
 
-static gf_boolean_t
+static int
 client_set_remote_options (char *value, xlator_t *this)
 {
         char         *dup_value       = NULL;
@@ -1176,56 +1172,53 @@ client_set_remote_options (char *value, xlator_t *this)
         char         *remote_port_str = NULL;
         char         *tmp             = NULL;
         int           remote_port     = 0;
-        gf_boolean_t  ret             = _gf_false;
+        int           ret             = 0;
 
         dup_value = gf_strdup (value);
         host = strtok_r (dup_value, ":", &tmp);
         subvol = strtok_r (NULL, ":", &tmp);
         remote_port_str = strtok_r (NULL, ":", &tmp);
 
-        if (!subvol) {
-                gf_msg (this->name, GF_LOG_WARNING, EINVAL,
-                        PC_MSG_INVALID_ENTRY, "proper value not passed as "
-                        "subvolume");
-                goto out;
+        if (host) {
+                host_dup = gf_strdup (host);
+                if (!host_dup) {
+                        goto out;
+                }
+                ret = dict_set_dynstr (this->options, "remote-host", host_dup);
+                if (ret) {
+                        gf_msg (this->name, GF_LOG_WARNING, 0, PC_MSG_DICT_SET_FAILED,
+                                "failed to set remote-host with %s", host);
+                        goto out;
+                }
         }
 
-        host_dup = gf_strdup (host);
-        if (!host_dup) {
-                goto out;
+        if (subvol) {
+                subvol_dup = gf_strdup (subvol);
+                if (!subvol_dup) {
+                        goto out;
+                }
+
+                ret = dict_set_dynstr (this->options, "remote-subvolume", subvol_dup);
+                if (ret) {
+                        gf_msg (this->name, GF_LOG_WARNING, 0, PC_MSG_DICT_SET_FAILED,
+                                "failed to set remote-host with %s", host);
+                        goto out;
+                }
         }
 
-        ret = dict_set_dynstr (this->options, "remote-host", host_dup);
-        if (ret) {
-                gf_msg (this->name, GF_LOG_WARNING, 0, PC_MSG_DICT_SET_FAILED,
-                        "failed to set remote-host with %s", host);
-                goto out;
+        if (remote_port_str) {
+                remote_port = atoi (remote_port_str);
+
+                ret = dict_set_int32 (this->options, "remote-port",
+                                      remote_port);
+                if (ret) {
+                        gf_msg (this->name, GF_LOG_ERROR, 0, PC_MSG_DICT_SET_FAILED,
+                                "failed to set remote-port to %d", remote_port);
+                        goto out;
+                }
         }
 
-        subvol_dup = gf_strdup (subvol);
-        if (!subvol_dup) {
-                goto out;
-        }
-
-        ret = dict_set_dynstr (this->options, "remote-subvolume", subvol_dup);
-        if (ret) {
-                gf_msg (this->name, GF_LOG_WARNING, 0, PC_MSG_DICT_SET_FAILED,
-                        "failed to set remote-host with %s", host);
-                goto out;
-        }
-
-        remote_port = atoi (remote_port_str);
-        GF_ASSERT (remote_port);
-
-        ret = dict_set_int32 (this->options, "remote-port",
-                              remote_port);
-        if (ret) {
-                gf_msg (this->name, GF_LOG_ERROR, 0, PC_MSG_DICT_SET_FAILED,
-                        "failed to set remote-port to %d", remote_port);
-                goto out;
-        }
-
-        ret = _gf_true;
+        ret = 0;
 out:
         GF_FREE (dup_value);
 
@@ -1252,11 +1245,6 @@ client_setxattr (call_frame_t *frame, xlator_t *this, loc_t *loc, dict_t *dict,
                 gf_msg (this->name, GF_LOG_INFO, 0, PC_MSG_RPC_INIT,
                         "client rpc init command");
                 ret = client_set_remote_options (value, this);
-                if (ret) {
-                        (void) client_destroy_rpc (this);
-                        ret = client_init_rpc (this);
-                }
-
                 if (!ret) {
                         op_ret      = 0;
                         op_errno    = 0;
@@ -2146,6 +2134,45 @@ client_icreate (call_frame_t *frame,
         return 0;
 }
 
+int32_t
+client_put (call_frame_t *frame, xlator_t *this, loc_t *loc, mode_t mode,
+            mode_t umask, uint32_t flags, struct iovec *vector, int32_t count,
+            off_t off, struct iobref *iobref, dict_t *xattr, dict_t *xdata)
+{
+        int          ret  = -1;
+        clnt_conf_t *conf = NULL;
+        rpc_clnt_procedure_t *proc = NULL;
+        clnt_args_t  args = {0,};
+
+        conf = this->private;
+        if (!conf || !conf->fops)
+                goto out;
+
+        args.loc = loc;
+        args.mode = mode;
+        args.umask = umask;
+        args.flags  = flags;
+        args.vector = vector;
+        args.count  = count;
+        args.offset = off;
+        args.size   = iov_length (vector, count);
+        args.iobref = iobref;
+        args.xattr = xattr;
+        args.xdata = xdata;
+
+        client_filter_o_direct (conf, &args.flags);
+
+        proc = &conf->fops->proctable[GF_FOP_PUT];
+        if (proc->fn)
+                ret = proc->fn (frame, this, &args);
+out:
+        if (ret)
+                STACK_UNWIND_STRICT (put, frame, -1, ENOTCONN, NULL, NULL,
+                                     NULL, NULL, NULL);
+
+        return 0;
+}
+
 int
 client_mark_fd_bad (xlator_t *this)
 {
@@ -2198,7 +2225,7 @@ client_rpc_notify (struct rpc_clnt *rpc, void *mydata, rpc_clnt_event_t event,
         }
         case RPC_CLNT_CONNECT:
         {
-                conf->connected = 1;
+                conf->can_log_disconnect = 1;
                 // connect happened, send 'get_supported_versions' mop
 
                 gf_msg_debug (this->name, 0, "got RPC_CLNT_CONNECT");
@@ -2216,7 +2243,7 @@ client_rpc_notify (struct rpc_clnt *rpc, void *mydata, rpc_clnt_event_t event,
                 client_mark_fd_bad (this);
 
                 if (!conf->skip_notify) {
-                        if (conf->connected) {
+                        if (conf->can_log_disconnect) {
                                 if (!conf->disconnect_err_logged) {
                                         gf_msg (this->name, GF_LOG_INFO, 0,
                                                 PC_MSG_CLIENT_DISCONNECTED,
@@ -2251,12 +2278,13 @@ client_rpc_notify (struct rpc_clnt *rpc, void *mydata, rpc_clnt_event_t event,
                                         "CHILD_DOWN notify failed");
 
                 } else {
-                        if (conf->connected)
+                        if (conf->can_log_disconnect)
                                 gf_msg_debug (this->name, 0,
                                               "disconnected (skipped notify)");
                 }
 
                 conf->connected = 0;
+                conf->can_log_disconnect = 0;
                 conf->skip_notify = 0;
 
                 if (conf->quick_reconnect) {
@@ -2866,6 +2894,7 @@ struct xlator_fops fops = {
         .setactivelk = client_setactivelk,
         .icreate      = client_icreate,
         .namelink     = client_namelink,
+        .put          = client_put,
 };
 
 
