@@ -104,17 +104,27 @@ posix_symlinks_match (xlator_t *this, loc_t *loc, uuid_t gfid)
         handle_size = POSIX_GFID_HANDLE_SIZE(priv->base_path_length);
         dir_handle = alloca0 (handle_size);
 
-        snprintf (linkname_expected, handle_size, "../../%02x/%02x/%s/%s",
+        snprintf (linkname_expected, PATH_MAX, "../../%02x/%02x/%s/%s",
                   loc->pargfid[0], loc->pargfid[1], uuid_utoa (loc->pargfid),
                   loc->name);
 
         MAKE_HANDLE_GFID_PATH (dir_handle, this, gfid, NULL);
         len = sys_readlink (dir_handle, linkname_actual, PATH_MAX);
-        if (len < 0)
+        if (len < 0 || len == PATH_MAX) {
+                if (len == PATH_MAX) {
+                        errno = EINVAL;
+                }
+
+                if (errno != ENOENT) {
+                        gf_msg (this->name, GF_LOG_ERROR, errno,
+                                P_MSG_LSTAT_FAILED, "readlink[%s] failed",
+                                dir_handle);
+                }
                 goto out;
+        }
         linkname_actual[len] = '\0';
 
-        if (!strncmp (linkname_actual, linkname_expected, handle_size))
+        if (!strcmp (linkname_actual, linkname_expected))
                 ret = _gf_true;
 
 out:
@@ -213,20 +223,24 @@ posix_lookup (call_frame_t *frame, xlator_t *this,
                                 "lstat on %s failed",
                                 real_path ? real_path : "null");
                 }
+                entry_ret = -1;
                 if (loc_is_nameless(loc)) {
                         if (!op_errno)
                                 op_errno = ESTALE;
                         loc_gfid (loc, gfid);
                         MAKE_HANDLE_ABSPATH (gfid_path, this, gfid);
-                        op_ret = sys_lstat(gfid_path, &statbuf);
-                        if (op_ret == 0 && statbuf.st_nlink == 1) {
-                                gf_msg (this->name, GF_LOG_WARNING, ESTALE,
+                        ret = sys_stat(gfid_path, &statbuf);
+                        if (ret == 0 && ((statbuf.st_mode & S_IFMT) == S_IFDIR))
+                                /*Don't unset if it was a symlink to a dir.*/
+                                goto parent;
+                        ret = sys_lstat(gfid_path, &statbuf);
+                        if (ret == 0 && statbuf.st_nlink == 1) {
+                                gf_msg (this->name, GF_LOG_WARNING, op_errno,
                                         P_MSG_HANDLE_DELETE, "Found stale gfid "
                                         "handle %s, removing it.", gfid_path);
                                 posix_handle_unset (this, gfid, NULL);
                         }
                 }
-                entry_ret = -1;
                 goto parent;
         }
 
