@@ -77,6 +77,23 @@
                }                                                              \
         } while (0)
 
+/* Setting microseconds or nanoseconds depending on what's supported:
+   The passed in `tv` can be
+       struct timespec
+   if supported (better, because it supports nanosecond resolution) or
+       struct timeval
+   otherwise. */
+#if HAVE_UTIMENSAT
+#define SET_TIMESPEC_NSEC_OR_TIMEVAL_USEC(tv, nanosecs) \
+        tv.tv_nsec = nanosecs
+#define PATH_SET_TIMESPEC_OR_TIMEVAL(path, tv) \
+        (sys_utimensat (AT_FDCWD, path, tv, AT_SYMLINK_NOFOLLOW))
+#else
+#define SET_TIMESPEC_NSEC_OR_TIMEVAL_USEC(tv, nanosecs) \
+        tv.tv_usec = nanosecs / 1000
+#define PATH_SET_TIMESPEC_OR_TIMEVAL(path, tv) \
+        (lutimes (path, tv))
+#endif
 
 #define GFID_NULL_CHECK_AND_GOTO(frame, this, loc, xattr_req, op_ret,         \
                                  op_errno, out)                               \
@@ -245,9 +262,11 @@ struct posix_private {
         uint32_t max_hardlinks;
 
         gf_boolean_t fips_mode_rchecksum;
+        gf_boolean_t ctime;
 };
 
 typedef struct {
+        call_frame_t *frame;
         xlator_t    *this;
         const char  *real_path;
         dict_t      *xattr;
@@ -310,17 +329,19 @@ int __posix_inode_ctx_get_all (inode_t *inode, xlator_t *this,
 
 int posix_gfid_set (xlator_t *this, const char *path, loc_t *loc,
                     dict_t *xattr_req);
-int posix_fdstat (xlator_t *this, int fd, struct iatt *stbuf_p);
-int posix_istat (xlator_t *this, uuid_t gfid, const char *basename,
-                 struct iatt *iatt);
-int posix_pstat (xlator_t *this, uuid_t gfid, const char *real_path,
-                 struct iatt *iatt);
+int posix_fdstat (xlator_t *this, inode_t *inode, int fd, struct iatt *stbuf_p);
+int posix_istat (xlator_t *this, inode_t *inode, uuid_t gfid,
+                 const char *basename, struct iatt *iatt);
+int posix_pstat (xlator_t *this, inode_t *inode, uuid_t gfid,
+                 const char *real_path, struct iatt *iatt,
+                 gf_boolean_t inode_locked);
 dict_t *posix_xattr_fill (xlator_t *this, const char *path, loc_t *loc,
                           fd_t *fd, int fdnum, dict_t *xattr, struct iatt *buf);
 int posix_handle_pair (xlator_t *this, const char *real_path, char *key,
                        data_t *value, int flags, struct iatt *stbuf);
-int posix_fhandle_pair (xlator_t *this, int fd, char *key, data_t *value,
-                        int flags, struct iatt *stbuf);
+int posix_fhandle_pair (call_frame_t *frame, xlator_t *this, int fd, char *key,
+                        data_t *value, int flags, struct iatt *stbuf,
+                        fd_t *_fd);
 void posix_spawn_janitor_thread (xlator_t *this);
 int posix_get_file_contents (xlator_t *this, uuid_t pargfid,
                              const char *name, char **contents);
@@ -350,6 +371,8 @@ posix_get_ancestry (xlator_t *this, inode_t *leaf_inode,
                     gf_dirent_t *head, char **path, int type, int32_t *op_errno,
                     dict_t *xdata);
 int
+posix_handle_mdata_xattr (call_frame_t *frame, const char *name, int *op_errno);
+int
 posix_handle_georep_xattrs (call_frame_t *, const char *, int *, gf_boolean_t);
 int32_t
 posix_resolve_dirgfid_to_path (const uuid_t dirgfid, const char *brick_path,
@@ -614,7 +637,30 @@ int32_t
 posix_rchecksum (call_frame_t *frame, xlator_t *this,
                  fd_t *fd, off_t offset, int32_t len, dict_t *xdata);
 
+int32_t
+posix_put (call_frame_t *frame, xlator_t *this, loc_t *loc,
+           mode_t mode, mode_t umask, uint32_t flags,
+           struct iovec *vector, int32_t count, off_t offset,
+           struct iobref *iobref, dict_t *xattr, dict_t *xdata);
+
 int32_t
 posix_set_mode_in_dict (dict_t *in_dict, dict_t *out_dict,
                         struct iatt *in_stbuf);
+
+gf_cs_obj_state
+posix_cs_check_status (xlator_t *this, const char *realpath, int *fd,
+                       struct iatt *buf);
+
+int
+posix_cs_set_state (xlator_t *this, dict_t **rsp, gf_cs_obj_state state,
+                    char const *path, int *fd);
+
+gf_cs_obj_state
+posix_cs_heal_state (xlator_t *this, const char *path, int *fd,
+                     struct iatt *stbuf);
+int
+posix_cs_maintenance (xlator_t *this, fd_t *fd, loc_t *loc, int *pfd,
+                   struct iatt *buf, const char *realpath, dict_t *xattr_req,
+                   dict_t **xattr_rsp, gf_boolean_t ignore_failure);
+
 #endif /* _POSIX_H */
